import 'dart:async';

import 'package:tencent_im_sdk_platform_interface/tencent_im_sdk_platform_interface.dart';
import 'package:tencent_im_sdk_plugin/enum/V2TimSDKListener.dart';
import 'package:tencent_im_sdk_plugin/models/v2_tim_callback.dart';
import 'package:tencent_im_sdk_plugin/models/v2_tim_user_full_info.dart';
import 'package:tencent_im_sdk_plugin/models/v2_tim_value_callback.dart';
import 'package:tencent_im_sdk_plugin/tencent_im_sdk_plugin.dart';

class TIMSdkMobile extends TIMSdkPlatform {
  @override
  Future<void> init({
    required int appId,
    TIMLogLevel logLevel = TIMLogLevel.off,
  }) async {
    Completer completer = Completer();

    V2TimValueCallback<bool> res = await TencentImSDKPlugin.v2TIMManager.initSDK(
      sdkAppID: appId,
      loglevel: logLevel.index,
      listener: V2TimSDKListener(
        onConnecting: () {
          eventStreamController.add(NetworkStatusChangedEvent(networkStatus: TIMNetworkStatus.connecting));
        },
        onConnectSuccess: () {
          eventStreamController.add(NetworkStatusChangedEvent(networkStatus: TIMNetworkStatus.connected));
          completer.complete();
        },
        onConnectFailed: (int code, String message) {
          eventStreamController.add(NetworkStatusChangedEvent(networkStatus: TIMNetworkStatus.connectFailed));
          completer.completeError(ConnectFailException(code, message));
        },
        onKickedOffline: () {
          eventStreamController.add(KickedOfflineEvent());
        },
        onUserSigExpired: () {
          eventStreamController.add(UserSignExpiredEvent());
        },
        onSelfInfoUpdated: (V2TimUserFullInfo info) {},
      ),
    );

    if (!(res.data ?? false)) {
      throw (InitSdkFailException());
    }

    return completer.future;
  }

  @override
  Future<void> login({required String userId, required String userSign}) async {
    V2TimCallback res = await TencentImSDKPlugin.v2TIMManager.login(
      userID: userId,
      userSig: userSign,
    );

    if (res.code != 0) {
      throw LoginFailException(res.code, res.desc);
    }
  }

  @override
  Future<void> logout() async {
    await TencentImSDKPlugin.v2TIMManager.logout();
  }

  @override
  Future<String> get sdkVersion async {
    V2TimValueCallback<String> result = await TencentImSDKPlugin.v2TIMManager.getVersion();
    return result.data ?? '';
  }

  @override
  Future<void> unInit() async {
    await TencentImSDKPlugin.v2TIMManager.unInitSDK();
  }
}
